/////////////////////////////////////////////////////////////////////////////////////
// File        : bp2595demo.c
// Function    : Simple demo for CPS model BP2595
// Copyright   : Copyright (C) 2021 CPS
/////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <conio.h>
#include <windows.h>

#include "apiBP2595.h"

static void rampHv(int devaddr);    // forward declaration

////////////////////////////////////////////////////////
// Main function.

int main(int argc, char *argv[])
{
    int devaddr = 0;        // Address of BP2595 we want to talk to
    int devflags;           // Detected device flags (ignored)
    BP2595_ERR errcode;     // Error code returned from API functions
    BP2595_STATUS status;   // BP2595 status info

    if (BP2595_OpenApi(&devflags) != BP2595_ERR_OK) {       // Open the API
        printf("Can't open BP2595 API\n");
        return 0;
    }

    errcode = BP2595_OpenDevice(devaddr, &status);          // Open the BP2595
    if (errcode == BP2595_ERR_OK) {

        if (status.StatusSystem.Live & STATE_REMOTE)        // If BP2595 is operating in remote control mode
            rampHv(devaddr);                                //   ramp the output voltage.
        else
            printf("BP2595 is not in remote control mode; terminating demo\n");

        BP2595_CloseDevice(devaddr);                        // Close the BP2595
    }
    else {
        printf("Error: %s\n", BP2595_ErrString(errcode));

    }
    BP2595_CloseApi();                                      // Close the API

    printf("Hit any key to exit ...");
    _getch();

    return 0;
}

////////////////////////////////////////////////////////////////////
// Ramp the HV output.

static void rampHv(int devaddr)
{
    s32 setpoint;           // Desired output voltage in volts
    BP2595_ERR errcode;     // Error code returned from API functions
    BP2595_METERS meters;   // Meter data

    errcode = BP2595_ProgramHvEnable(devaddr, 1);               // Enable HV output.
    if (errcode != BP2595_ERR_OK) {
        printf("Can't enable HV output: %s\n", BP2595_ErrString(errcode));
        return;
    }

    errcode = BP2595_SetMeterInterval(devaddr, 100);            // Turn on the meter stream -- tell BP2595 to send meter data every 100 ms.
    if (errcode != BP2595_ERR_OK) {
        printf("Can't start the meter stream: %s\n", BP2595_ErrString(errcode));
        return;
    }

    printf("Starting ramp\n");

    for (setpoint = -5000; setpoint < 5000; setpoint += 100)    // Ramp the HV output from -5 to +5 kV in 100 V steps -----------------
    {
        errcode = BP2595_ProgramSetpoint(devaddr, setpoint);        // Program the output voltage.
        if (errcode != BP2595_ERR_OK) {
            printf("Can't program output voltage: %s\n", BP2595_ErrString(errcode));
            break;
        }

        Sleep(500);                                                 // Delay approximately 0.5 seconds

        errcode = BP2595_GetMeterUpdate(devaddr, &meters, 0);       // Fetch and display the most recent meter data.
        if (errcode != BP2595_ERR_OK) {
            printf("Failed to receive meter data: %s\n", BP2595_ErrString(errcode));
            break;
        }
        printf("setpoint = %d; meters = %d V, %d uA\n", setpoint, meters.outV, meters.outUA);
    }

    if (errcode == BP2595_ERR_OK)
        printf("Ramp completed\n");
    else
        printf("Ramp aborted\n");

    BP2595_ProgramHvEnable(devaddr, 0);                         // Turn off HV and meter stream.
    BP2595_ProgramSetpoint(devaddr, 0);
    BP2595_SetMeterInterval(devaddr, 0);
}

